#!/usr/bin/perl 
use Carp;
use English;
use Cwd;
use Getopt::Std;

# Gen2ril_v1Std.pl is toggled for standard time (no DLS) see line ~326
# NOTE!!! Check to be sure date::calc is commented out.
# Gen2ril_v1.pl converts csv weather data file to csv file ("ril" file) with calculation results and format to run STEMRUST_G rust model

# Gen2ril_v1.pl is identical to Gen2ril_draft2.pl, except for removing some comments from the top of the file 
# Gen2ril_draft2.pl (general format to ril) has added code to get temperature at sunrise ($SRtemp). 
#  Also added code for DLS time in southern hemisphere, and hard-coded switch for Europe or USA DLS.
# Command-line option to select canopy-level or standard-height temperature measurements, default is canopy ht
# This program uses RH>98 as a redundant criterion for wetness (is hard-coded, and can be changed).
# The ril file produced by this version has sunrise time in the 4th column 
# In calculating sunrise(set) the code from util-rust_v2.pl uses command-line opt for hrs from GMT, for SR_SS calcs away from PNW longitudes
#   opts: opt_s for station name, opt_t latitude, opt_g longitude, opt_h hrs from GMT zone (defaults are for Corvallis, Oregon, USA)
# Note convention for longitude sign in sunrise/set calc, i.e. east long is negative, west long is positive; hours east of GMT are negative
#      south latitude is negative, north is positive


# pull in the utility subroutines.
BEGIN {                       
        require "utilities_pruned.pl";   # if "utilities-pruned.pl" not available, can use "utilities.pl" instead
#       require "utilities.pl"; 
}


# following is a list of switches that can be used in the command line for data input options
# process flags on command line  
($opt_c,$opt_s,$opt_t,$opt_g,$opt_h) = (undef,undef,undef,undef,undef);
getopts('c:s:t:g:h:');  # a colon means there is an argument for the option

# NOTE: option for temperature measurement type in input data: standard-ht T only (0.1) or canopy-level T (1)  in input data
  $CanopyData = $opt_c ? $opt_c : 1; #default (1) if there IS canopy-level temperature data, set opt_c to 0.1 if there isn't
# FIXME need a warning if weather input file has no canopy level data but opt_c set or default to 1 (all results are 0)

##### Station-specific information ####
# on the command line, -s sets station ID, -t sets lattitude, -g sets longitude (NOTE neg longi values for E)

# set station ID number in command line, or use a default value
  $StationID = $opt_s ? $opt_s : $locID;

# set station lattitude in command line, or use a default value (for Hyslop location)
 $Lat = $opt_t ? $opt_t : 44.63;  

# set station longitude in command line, or use a default value (for Hyslop location)
 $Long = $opt_g ? $opt_g : 123.20; #unlike usual convention, neg values indicate E longitude

# set time zone (hrs from GMT)  in command line, or use a default value (for PNW longitudes)
 $delta_hr_GMT = $opt_h ? $opt_h : 8; #PNW (default) is +8 hr from GMT zone, use neg number for eastern longitudes 

# notify which temperature data and calculations will be used
if($CanopyData == 1){
   print STDERR "InfValues and LUP calculated from canopy-height temperatures
   (returns zeros if there is no canopy-height temperature data). \n";
}
elsif($CanopyData < 0.5){
   print STDERR "InfVals and LUP calculated from standard-height temperatures, then adjusted before output. \n";
}


##### subroutines #####
sub setinterval( $ );
sub reset_stats;
sub sin_deg ($);
sub atan_deg ($);
sub square ( $ );
sub power ( $$ );
sub sun_calc ($$$$);
sub LUP_per_hour($);
#sub dumphash ($$);
sub hashify_line ($);
sub get_daylight_saving_start_doy;
sub get_daylight_saving_end_doy;
sub get_march_day;
sub get_april_day;
sub get_october_day;
sub get_november_day;

# set the data interval.  This should only need to be done once per run,
# but we do it for every 15- or 30- minute line because it's easier
sub setinterval( $ ) {
    $interval = shift @_;        # data interval in minutes
    $interval_hr = $interval/60; # data interval in hours
}

sub reset_stats {
    # reset the cumulative stats 
    $NiteWet   = 0;
    $NWdh_cumu = 0;
    $SrWdh_cum = 0;
    $WetIntrup = 1;  # default value
}

##### Vars and Initialization #####

#$sun_calc_day:  day-of-year for which sun{rise,set} times were last calculated
#$reported_day:  day-of-year for which last report was generated

$first_pass = 1;  # flag for initialization that based on 1st data line
reset_stats();


######################### following block (to next ######...) was brought in from rustMIN.conf


# -----field information for input weather data------------------------------
# This will handle three kinds of lines: 15, 30 or 60-minute data.
# Each type has the same format (headers and column order), just differing in first column
# Need to have at least the first 8 columns (thru Tstd) in input file

#fields and format strings for "15" records (every 15 minutes)
#revised 11Feb 2014, for non-LUP input
@tmp = (
	"15Min"            =>     "%3d    ",
	"loc ID"           =>     "%s  ",       # weather station ID
	"Year"	           =>     "%4d    ",
	"Day"	           =>     "%3d    ",    # day-of-year (doy), 1 - 365(366)
	"Time"	           =>     "%4d    ",    # format: eg 10 am is 1000, 2:30 pm is 1430
	"Wetness"          =>     "%1d    ",    # boolean; 1 is wet
	"Rain"             =>     "% 5.2f ",    # mm rain summed over previous day
	"Tstd"             =>     "% 5.2f ",    # Temperature (C) measured at standard height (~1.5 m) 
	"Tcnpy"            =>     "% 5.2f ",    # Temperature (C) measured at crop canopy-height
	"RHstd"            =>     "% 5.2f ",    # RH (90% = 90) measured at standard height (~1.5 m)
	"Wind mps"         =>     "% 5.2f "     # not used now but could be (later) for LeafWetness algorithm
	);
@campbell_fields_15 = ();
%campbell_formats_15 = ();
while(@tmp) {
    my $field = shift @tmp;
    my $format = shift @tmp;
    push @campbell_fields_15, $field;
    $campbell_formats_15{$field} = $format;
}

#fields and format strings for "30" records (every 30 minutes)
#revised 11Feb 2014, for non-LUP input
@tmp = (
	"30Min"            =>     "%3d    ",
	"loc ID"           =>     "%s  ",   
	"Year"	           =>     "%4d    ",
	"Day"	           =>     "%3d    ",
	"Time"	           =>     "%4d    ",
	"Wetness"          =>     "%1d    ",    # boolean; 1 is wet
	"Rain"             =>     "% 5.2f ",
	"Tstd"             =>     "% 5.2f ",
	"Tcnpy"            =>     "% 5.2f ",
	"RHstd"            =>     "% 5.2f ",
	"Wind mps"         =>     "% 5.2f "
	);
@campbell_fields_30 = ();
%campbell_formats_30 = ();
while(@tmp) {
    my $field = shift @tmp;
    my $format = shift @tmp;
    push @campbell_fields_30, $field;
    $campbell_formats_30{$field} = $format;
}


#fields and format strings for "60" records (every 60 minutes)
#revised 11Feb 2014, for non-LUP input
@tmp = (
	"60Min"            =>     "%3d    ",
	"loc ID"           =>     "%s  ",   
	"Year"	           =>     "%4d    ",
	"Day"	           =>     "%3d    ",
	"Time"	           =>     "%4d    ",
	"Wetness"          =>     "%1d    ",    # boolean; 1 is wet
	"Rain"             =>     "% 5.2f ",
	"Tstd"             =>     "% 5.2f ",
	"Tcnpy"            =>     "% 5.2f ",
	"RHstd"            =>     "% 5.2f ",
	"Wind mps"         =>     "% 5.2f "
	);
@campbell_fields_60 = ();
%campbell_formats_60 = ();
while(@tmp) {
    my $field = shift @tmp;
    my $format = shift @tmp;
    push @campbell_fields_60, $field;
    $campbell_formats_60{$field} = $format;
}
undef @tmp;

%inverse_fields_15 = ();
for($i=0; $i<=$#campbell_fields_15; $i++) {
    $inverse_fields_15{$campbell_fields_15[$i]} = $i;
}

%inverse_fields_30 = ();
for($i=0; $i<=$#campbell_fields_30; $i++) {
    $inverse_fields_30{$campbell_fields_30[$i]} = $i;
}
%inverse_fields_60 = ();
for($i=0; $i<=$#campbell_fields_60; $i++) {
    $inverse_fields_60{$campbell_fields_60[$i]} = $i;
}

# make these available in indexed form 
$fields{15} = \@campbell_fields_15;
$fields{30} = \@campbell_fields_30;
$fields{60} = \@campbell_fields_60;
$inverse_fields{15} = \%inverse_fields_15;
$inverse_fields{30} = \%inverse_fields_30;
$inverse_fields{60} = \%inverse_fields_60;


######################### end of code brought in from rustMIN.conf


####################### following block (to next #######....) brought in from util-rustMIN.pl

# util-rust_v2.pl (8Jan2014) uses an input for hrs from GMT, for SR_SS calcs away from PNW longitudes
# note convention for longitude sign in sunrise/set calc, i.e. east long is negative, west long is positive

# use Date::Calc qw(Day_of_Year);

$DEG_PER_RADIAN = 45.0 / atan2(1,1);
sub sin_deg ($) {
    my $deg = shift;
    return sin($deg/$DEG_PER_RADIAN);
}

sub atan_deg ($) {
    my $tan = shift;
    my $atan = $DEG_PER_RADIAN * atan2($tan, 1);
    # perl's atan2 returns a result in -pi to pi, so -90 < $atan < 90
    $atan += 180 if $atan<0;  # now 0 < $atan < 180, which we want.
    return $atan;
}

sub square ( $ ) {
    my $x = shift @_;
    return $x*$x;
}

sub power ( $$ ) {
    my ($x, $y) = @_;
    return exp( $y*log($x) );
}


$Const15 = 15.0; # earth's rate of rotation in degrees per hour

# inputs: are $Day (of year), $Lat, $Long
sub sun_calc ($$$$) {      #note: 4 inputs (incl delta_GMT) in util-rust_v2.pl or util-rustMIN_v2.pl, only 3 in util-rust.pl
    my ($Day,
        $Lat,
        $Long,
        $delta_hr_GMT, #location's time zone in hrs (integer) from GMT, negative number for east Longitudes 
        $year,    # needed for daylight saving
        ) = @_;

    my $dy_100 = 0.01*$Day;
    my $dy180_100 = 0.01*($Day-180);

    # calculate declination of sun as a function of date (approximation to
    # a far more elaborate calculation or perhaps to measurements
    my $sind = polynomial($dy_100,
			  # c0        c1      c2       c3      c4       c5
			  -.37726, -.10564, 1.2458, -.75478, .13627, -.00572);
    my $cosd = sqrt(1.0-$sind*$sind);

    my $alt = -0.83333;  # altitude at set/rise (50 arcmin below horizon)
    # a bit below the horizon -- atmospheric refraction 

    # calculate cos H and H  -- a bunch of spherical trig.
    # H appears to be angle through which the earth rotates between sunrise
    # and local solar noon.  That is, it is the angle along the appropriate
    # arc of constant declination (think of parallels in lat-long) between
    # the sun's position at sunrise and the sun's position at zenith.
    my $proc1 = sin_deg($alt);
    my $sinlat = sin_deg($Lat);
    my $coslat = sqrt(1.0-$sinlat*$sinlat);
    my $proc2 = -$sind*$sinlat;
    $proc1 += $proc2;
    $proc2 =  $cosd*$coslat;
    my $cosH = $proc1/$proc2; #(sin(alt)-sin(d)*sin(lat))/(cos(d)*cos(lat)) ???
    my $sinH = sqrt(1-$cosH*$cosH);
    my $H = atan_deg($sinH/$cosH);

    my $hrangle = $H/$Const15;  # units of hours
    my $hrrise = 12-$hrangle;   # decimal hours after midnight
    my $hrset = 12+$hrangle;    # ditto

    # longitude correction
    # Note!!!: west longitude is positive, use negative number for east longitude
    # standard meridian for the time zone. Zone (hr from GMT) is set in norm2ril command line, else defaults to PNW value 
    $SMerid = 15 * $delta_hr_GMT;   
    my $LCorr = ($SMerid - $Long)/$Const15;

    # equation of time
    # difference between solar time and mean time, aka nonlinearities in the
    # sun's right ascension, or horiz. part of the figure-8 the sun traces.
    my $EqTime = 0;  # declare + silence warnings
    if ($Day < 180.5) {
	$EqTime = polynomial($dy_100, 
			     -.04056, -.74503, .08823,
			     2.0516, -1.8111, 0.42832);
    } else {
	$EqTime = polynomial($dy180_100, 
			     -0.05039, -.33954, 0.04084,
			     1.8928, -1.7619, 0.4224);
    }

    # calculate corrected rise
    $proc1 = $hrrise - $LCorr;
    my $SunR_hrs = $proc1 - $EqTime;
    my $SunR_hour = int($SunR_hrs);
    my $SunR_min = 60*($SunR_hrs - $SunR_hour);
    my $SunR_hrmn = 100*$SunR_hour + $SunR_min;
    #$SunRise = $SunR_hrmn;

    # calculate corrected set
    $proc1 = $hrset - $LCorr;
    my $SunS_hrs = $proc1 - $EqTime;
    my $SunS_hour = int($SunS_hrs);
    my $SunS_min = 60*($SunS_hrs - $SunS_hour);
    my $SunS_hrmn = 100*$SunS_hour + $SunS_min;
    #$SunSet = $SunS_hrmn;

    # Choose whether or not to account for daylight saving time 
     if(0) {            #uncomment this line to omit DLS from SunR_SunS calc, just use standard time
#     if(1) {            #to keep DLS, uncomment this line instead
         if($Lat > 0){  # for northern latitudes; comment/uncomment following line for Europe vs USA
#           if ($Day >= 87 and $Day < 300) { # for Europe - these dates should be correct within 3 days
            if ($Day >= 70 and $Day < 306) { # for USA - these dates should be correct within 3 days
	           $SunR_hrmn += 100;
	           $SunS_hrmn += 100;
            }
         }
     
         elsif($Lat<0){ #for southern latitudes - using New Zealand dates here - modify as needed
            if ($Day >= 270 and $Day < 94) { # for New Zealand - these dates should be correct within 3 days
	           $SunR_hrmn += 100;
	           $SunS_hrmn += 100;
            }
         }

      }   # end bracket for choice of if(0) or if(1)

    return $SunR_hrmn, $SunS_hrmn;

} # end of sub sun_calc

#  moved sub LUP_per_hour downward from this location, to just before other LUP calcs 

sub hashify_line ($) {

    local $_ = shift @_;
    chomp;
    my $href = {};
    my $type;

    my @pieces = split /,/;
    if ($pieces[0] == 15) {    # this line and next added in MIN version
	$type=15;
    } elsif ($pieces[0] == 30) {   
	$type=30;
    } elsif ($pieces[0] == 60) {
	$type=60;
    } else {
	die "unknown line type $pieces[0] in line $INPUT_LINE_NUMBER";
    }
#    print "type $type from $pieces[0]\n";

    if ($#pieces < $#{$fields{$type}}) {
	warn "Line $INPUT_LINE_NUMBER is incomplete -- skipping";
	warn "[$_]";
	return undef();
    }

    my @flist = @{$fields{$type}};
    @$href{@flist} = @pieces;

    # fix persistent issue with leading zeros in the time
    $href->{"Time"} = substr('0000'.$href->{"Time"}, -4, 4);
    return $href, $type;
}

sub get_daylight_saving_start_doy {
   my $year = shift;
   my $day;
   my $month;
   if ($year > 2006) {
      $day =  &get_march_day($year);
      $month = '03';
   }
   else {
      $day = &get_april_day($year);
      $month = '04';
   }
   my $doy = &Day_of_Year($year, $month, $day);
   return $doy;
}

sub get_daylight_saving_end_doy {
   my $year = shift;

   my $day;
   my $month;
   if ($year > 2006) {
      $day = &get_november_day($year);
      $month = '11';
   }
   else {
      $day = &get_october_day($year);
      $month = '10';
   }
   my $doy = &Day_of_Year($year, $month, $day);
   return $doy;
}

sub get_march_day {
   my $year = shift;
   my $march_date = 14 - (int(1 + $year * 5/4) % 7);
   return $march_date;
}

sub get_april_day {
   my $year = shift;
   my $april_date = (2 + 6 * $year - int($year/4) ) % 7 +1;
   return $april_date;
}
sub get_october_day {
   my $year = shift;
   my $october_date = (31 - (int($year * 5/4) +1) % 7);
   return $october_date;
}
sub get_november_day {
   my $year = shift;
   my $november_date = 7 - (int( 1 + $year * 5/4) % 7);
   return $november_date;
}


####################### end of block brought in from util-rustMIN.pl


##### Loop Start for creating ril file #####
# read, parse, and process input weather data

while(<>) {
    # parse the 15- 30 or 60-minute records in weather data file
    chomp;
    $href = {};
    $type = -99;

    @pieces = split /,/;
    foreach $pc (@pieces) {   # remove leading and trailing whitespace
	$pc =~ s/^"//;
	$pc =~ s/^\s+//;
	$pc =~ s/\s+$//;
    }
    #print join('; ',@pieces), "\n";
   if ($pieces[0] == 15) {
	$type=15;
	setinterval(15);
   } elsif ($pieces[0] == 30) {
	$type=30;
	setinterval(30);
   } elsif ($pieces[0] == 60) {
	$type=60;
	setinterval(60);
    } else {
	carp "unknown line type $pieces[0] in line $INPUT_LINE_NUMBER";
    }

    #if ($#pieces < $#{$fields{$type}}) {
    if ($#pieces < 8) {  #need to have columns at least thru Tstd
	warn "Line $INPUT_LINE_NUMBER is incomplete -- skipping";
	next;
    }
    foreach $field (@{$fields{$type}}) {
	$$href{$field} =  shift @pieces;
    }

    # set vars from data
    $locID = $href->{'loc ID'};         # weather station ID 
    $Year_1	= $href->{'Year'};     
    $Day	= $href->{'Day'};       # day-of-year (doy), 1 - 365(366)
    $Time	= $href->{'Time'};	# format: eg 10 am is 1000, 2:30 pm is 1430 (weather station reads its clock)
    $LeafWet	= $href->{'Wetness'};   # boolean; 1 is wet, 0 not wet
    $Rain_mm	= $href->{'Rain'};      # mm rain summed over previous day
    $Tstd	= $href->{'Tstd'};      # Temperature (C) measured at standard height (~1.5 m) 
    $AvgT	= $href->{'Tcnpy'};     # Temperature (C) measured at crop canopy-height
    $RHstd	= $href->{'RHstd'};     # RH (format:90% coded as 90) measured at standard height (~1.5 m)
    $Wndspd	= $href->{'Wind mps'};  # not used now but could be (later) for LeafWetness algorithm


    if ($CanopyData < 0.5){  # if using standard-height temperature data, rather than canopy-level temps ($opt_c)
       $AvgT = $Tstd;
    }
 
     # alter time format of midnight
    if ($Time eq "2400") {  # preferred midnight convention
	$Time = '0000';
	$Day += 1;
    }

    # Extra stuff for the first pass
    if ($first_pass) {
	# do sunset calc a bit early
	($SunRise, $SunSet) = sun_calc($Day, $Lat, $Long, $delta_hr_GMT);
	$sun_calc_day = $Day;

	# start reporting after the first complete night
	$reported_day = ($Time < $SunSet) ? $Day : $Day+1;

        print STDERR "Using input data with $type minute resolution. \n";

	$first_pass = 0;
    }


    # if it's a new day, recalc rise and set times
    if ($Day != $sun_calc_day) {
	($SunRise, $SunSet) = sun_calc($Day, $Lat, $Long, $delta_hr_GMT);
	$sun_calc_day = $Day;
#     print STDERR "day $Day sunrise $SunRise sunset $SunSet \n";  #debug to check sunrise(set) calcs
   }

#  get temperature at sunrise - not needed for ril, but useful data
   if ($Time > $SunRise and hhmm_diff($Time,$SunRise) <= $interval_hr) {
       $SRtemp = $AvgT;
#       print STDERR "day $Day Time $Time SunRise $SunRise   interval $interval    SRtemp $SRtemp \n";
   }

    # Is it night or day?
    $Day_Nite = ($Time > $SunRise and $Time <= $SunSet) ? 1 : 0;
    # is it in the first two hours after sunrise?
    # original line follows. I extended the dawn time to better match CSI InfVal output [Bill]
#   $Dawn = ($Time > $SunRise and $Time <= $SunRise + 200) ? 1 : 0;
    $Dawn = ($Time > $SunRise and $Time <= $SunRise + 215) ? 1 : 0;

    # accumulate rainfall
    $rain_today += $Rain_mm;

#  following sub was originally further up in the code 
# Calculate LUP units per hour. 
sub LUP_per_hour($) {
    my $t = shift @_;  
    my $t_pos = max($t,0); 
    # First, calculate LUP per half hour (because the constants we know are
    # for half-hours.
    my $LUP_30min = 0.0001560*$t_pos  + -2.060e-4;
    my $Proc = 1.0- exp( 0.4970*( $t_pos - 35.5 ) );
    $LUP_30min *= $Proc;

    # Now rescale it to LUP per hour
    my $LUP_per_hr = $LUP_30min * 2;

#    print STDERR "$LUP_per_hr\t$LUP_30min\t$Proc\t$t_pos\n"; #debug

    $LUP_per_hr = 0 if $LUP_per_hr < 0;
    return $LUP_per_hr;
}

    ## Calculate LUP (latent unit periods)
 #  print STDERR "interval is $interval and interval_hr  is $interval_hr \n";
    $LUP_interval = $interval_hr * LUP_per_hour($AvgT);
    $LUP_Tot = $LUP_interval + $LUP_Tot; # daily running total
    $LUP_Cum = $LUP_interval + $LUP_Cum; # cumulative over whole season

    ##### start calculations leading to infection value #####
    # Calculate Degree hours in this interval
    $DegOverTh = ($AvgT > 31.3) ? 0 : $AvgT - 2.0;
    $SmplDegHr = $DegOverTh*$interval_hr; # degree hours per interval
    $SmplDegHr = 0 if $SmplDegHr < 0;

    # ADJUST FOR D-H TEMP EFFECT (range of factor is 0 - 1)
    #   for CSI software have to calculate factor X 10000

    # Poly Coeffs described as C0-C5 respectively
    #$dhAdjFctr = .0001*polynomial($t, -9050, 6540, -812.6, 45, -1.145, 0.01064);
    $dhAdjFctr = polynomial($AvgT,
			    -0.9050,
			    0.6540,
			    -0.08126,
			    4.5e-3,
			    -1.145e-4,
			    1.064e-6);

    $dhAdjFctr = 0 if $dhAdjFctr < 0; # now redundant

    $TmpAdjDh = $SmplDegHr * $dhAdjFctr;  # corrected degree-hours/interval
    # Carry both SmplDegHr (simple, uncorrected) and TmpAdjDh (corrected)
    # degree hour values forward.

    # Aggregate wetness information from leaf wetness sensors, relative
    # humidity sensors, and a rain gauge.  This information is already
    # aggregated by our campbell dtalogger program, (so the B_ and C_test are redundant in the "or" test), 
    # but it would act to back up (with RH and rain) a leafwetness test alone if that is
    # what is recorded in the "wetness" column of input data.
    #  NOTE!!! that the following sets all time intervals w RH>98 as wet - recode if desired
    $A_Test = $LeafWet; #this is the 'wetness' column in Campbell data
    $B_Test = ($RHstd > 98)?1:0;   # NOTE this threshold; could write it as an option instead of hard-coded here?
    $C_Test = ($Rain_mm >= 0.1)?1:0;
    $Wet_ABC = ($A_Test or $B_Test or $C_Test)?1:0;

    ### Accumulate infection statistics ###
    if ($Day_Nite<0.5) {		# is it night?
	if ($Wet_ABC) {			    # and wet ?
	    # accumulate night wet hours
	    $NiteWet += $interval_hr;
	    # Night wet degree hours
	    $NWdh_cumu += $TmpAdjDh;
	} else {			# night and dry
	    # If rust is sensitive to interruption in wetness now,
	    if ($NWdh_cumu >= 5.5 and $NWdh_cumu < 15) {
		# less rust will survive (factor default is 1.0)
		$WetIntrup = 0.6;
	    }
	}
    } elsif ($Wet_ABC and $Dawn) {
	# accumulate wet degree-hours during $Dawn (the 2.25 hours after sunrise)
	$SrWdh_cum += $TmpAdjDh;
    }

    # at (or very soon after) midnight
    # -- should be 00:00, but campbell datalogger uses 24:00

     if ($Time eq '2400' or $Time < $interval) {
	  # record day's (at midnite) rain and season's LUP for later output
	  $saved_rain = $rain_today;
	  $saved_LUP_cumu = $LUP_Cum;

	  # the states which accumulate midnight to midnight, re-setting each day
	  $rain_today = 0;
	  $LUP_Tot = 0;
     }

    # Is it time to issue the output for today?
    if ($Day_Nite > 0.5              # if it's daytime
	and not $Dawn                # and the the dawn period is over
	and $reported_day != $Day) {  # and we haven't reported yet today, should work for doys in wrap-around year data  
#	and (($reported_day < $Day) or (($Day == 1) and ($Time == 1000))  )) {  # backup if needed after testing, otherwise delete 
	$reported_day = $Day;            # report

	# calculate the actual infection value
	$DgHrIndex = $NWdh_cumu * $SrWdh_cum * $WetIntrup;
#	print STDERR "$Day $SunRise $SunSet $DgHrIndex = $NWdh_cumu * $SrWdh_cum * $WetIntrup\n"; #debug
	# calculate relative degree-hr infection value (RIV), with 
	# values between 0 and 1, using the index.  
	$calcRIV = 1 - exp(-0.0031*$DgHrIndex);
	#$InfValPwr = -0.0031*$DgHrIndex;           # just noting the calculation steps 
	#$calcRIV = 1 - exp($InfValPwr);            # just noting the calculation steps
	# final rust infection value on 0-3 scale   # just noting the calculation steps
	$StdDhInfV = 3.0 * $calcRIV;


    if ($CanopyData < 0.5){    # see Plant Disease(2012)96:726-736. algorithms to convert standard-ht temp results to canpy equiv
        $saved_LUP_cumu_emul = $saved_LUP_cumu * 1.05;                   # can change multiplier, if have information for location/season
        $StdDhInfV_emul = 1*$StdDhInfV + 0*$StdDhInfV*$StdDhInfV;        # can change coefficients, if have information
 #       $StdDhInfV_emul = 0.85*$StdDhInfV + 0.05*$StdDhInfV*$StdDhInfV; # coefficients from sev yrs Hyslop data

    }


    # Finally, output  ($calcRIV or $StdDhInfV?)

    if ($CanopyData < 0.5) {
	print ("$StationID,$Year_1, $Day,");
	printf("%5.0f, %5.2f, %4.2f, %7.3f\n",
	       $SunRise, $saved_rain, $StdDhInfV_emul, $saved_LUP_cumu_emul); #puts sunrise time in ril file - not needed, but informative 
	reset_stats();

    }

    elsif ($CanopyData == 1) {
	print ("$StationID,$Year_1, $Day,");
	printf("%5.0f, %5.2f, %4.2f, %7.3f\n",
	       $SunRise, $saved_rain, $StdDhInfV, $saved_LUP_cumu); #puts sunrise time in ril file - not needed, but informative 
	reset_stats();
    }

}   
}




